CREATE OR REPLACE
PROCEDURE PR_ALTA_CLIENT (

p_idClient IN client.idClient%TYPE,
p_NIF IN client.NIF%TYPE,
p_tipusClient IN client.tipusClient%TYPE,
p_nom IN client.nom%TYPE,
p_cognom1 IN client.cognom1%TYPE,
p_cognom2 IN client.cognom2%TYPE,
p_adreca IN client.adreca%TYPE,
p_codiPostal IN client.codiPostal%TYPE,
p_localitat IN client.localitat%TYPE,
p_pais IN client.pais%TYPE,
p_dataAntiguitat IN client.dataAntiguitat%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CLIENT
--
--  Dona d'alta un client a la taula CLIENT.
--  Cada client t un idClient diferent, per poden haver-hi diversos idClients
--  que pertanyin a la mateixa empresa (NIF). Aix s ms real, ja que una
--  empresa pot tenir delegacions i cadascuna acta com un client diferent.
--  En cas que al donar d'alta hi hagi ms d'una entrada amb el mateix NIF,
--  el sistema ens avisa de tal circumstncia (per s'inserta el registre).
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la
--  TAULALOG el resultat de la operaci.
--
--
--  ENTRADA : p_IDCLIENT NUMBER(10), p_NIF CHAR(9), p_TIPUSCLIENT   CHAR(1),  
--    p_NOM VARCHAR2(20), p_COGNOM1 VARCHAR2(20), p_COGNOM2 VARCHAR2(20),  
--    p_ADRECA  VARCHAR(30), p_CODIPOSTAL  CHAR(5) , p_LOCALITAT VARCHAR2(30), 
--    p_PAIS  VARCHAR2(20), p_DATAANTIGUITAT  DATE.
--
--  SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.

--  El valor de idClient, si s zero, s'agafa de la seqncia s_idClient.
--  DataBaixa es posar a nul al donar d'alta.
--  Si tipusClient = 'E' els cognoms han de ser nulls.
--  Si tipusClient = 'P' almenys el primer cognom ha d'omplirse.
--
--    RSP = OK : S'inserta registre.
--
--    RSP = ERROR + motiu, si hi ha algn error.
--
--    Motius errors tractats:
--      - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--      - ERROR: IDCLIENT  JA EXISTEIX A LA TAULA
--              (NIF ES POT REPETIR, PER IDCLIENT NO)
--  	- ERROR: TIPUS CLIENT HA DE SER ''P'' = PERSONA O ''E'' = EMPRESA.
--	- ERROR: PERSONA HA DE TENIR COGNOM1 ALMENYS.
--      - ERROR: EMPRESA AMB COGNOMS.
--      - ERROR: DATA DE ALTA DE CLIENT SUPERIOR A LA ACTUAL.
--      - Altres.
--
--
--  NO S MOTIU D'ERROR que hi hagi registres amb NIF iguals
--
--******************************************************************************/


IS
    -- variable de la data baixa del client que s'insertar com a null
    dataBaixaCli  client.dataBaixa%TYPE := null;


    -- variables errors
    e_idClient_repetit EXCEPTION;
    e_camps_nuls EXCEPTION;
    e_tipusClient_erroni  EXCEPTION;
    e_persona_sense_cognom1 EXCEPTION;
    e_empresa_amb_cognoms EXCEPTION;
    e_dataAntiguitat_erronia EXCEPTION;

    -- variables insert taulaLOG
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CLIENT';
    paramEntrada taulaLog.paramEntrada%TYPE := 'p_idClient: ' || p_idClient ||', p_NIF: ' || p_NIF || ', p_tipusClient: ' || p_tipusClient ||
             ', Nom : ' || p_nom || ', cognom1 : ' || p_cognom1 || ', cognom2 : ' || p_cognom2 ||
            ', p_adreca: ' || p_adreca || ', p_codiPostal: ' || p_codiPostal || ', p_localitat: ' || p_localitat
            ||' ,p_pais: '|| p_pais|| ', p_dataAntiguitat: ' || p_dataAntiguitat;

    paramSortida taulaLog.paramSortida%TYPE := 'RSP';

    -- variables d's divers.
    numElements number :=0;
    numIdSeq number;

BEGIN

      -- VERIFIQUEM SI HI HA NULS ALS PARAMETRES D'ENTRADA
      IF ((p_idClient is null) or (p_NIF is null) or (p_tipusClient is null) or (p_nom is null) or (p_adreca is null) or (p_codiPostal is null) or
         ( p_localitat is null) or (p_pais is null) or (p_dataAntiguitat is null))
          THEN RAISE e_camps_nuls;
      END IF;

      --VERIFIQUEM QUE IDCLIENT NO ESTIGUI REPETIT
      SELECT COUNT(*) INTO numElements
        FROM CLIENT
        WHERE IDCLIENT = p_idClient;

      IF numElements > 0
       THEN RAISE e_idClient_repetit;
      END IF;

      -- VERIFIQUEM EL TIPUS DE CLIENT
      IF p_tipusClient <> 'E' and p_tipusClient <> 'P'
        THEN RAISE e_tipusClient_erroni;
      END IF;

      -- VERIFIQUEM QUE EMPRESA NO TINGUI COGNOMS
      IF p_tipusClient = 'E' and (p_cognom1 is not null or p_cognom2 is not null)
        THEN RAISE e_empresa_amb_cognoms;
      END IF;

      -- VERIFIQUEM QUE NOM TINGUI ALMENYS COGNOM 1.
      IF p_tipusClient = 'P' and p_cognom1 is null
        THEN RAISE e_persona_sense_cognom1;
      END IF;

      -- VERIFIQUEM QUE DATA ALTA SIGUI INFERIOR A LA ACTUAL
      IF p_dataAntiguitat > sysdate
        THEN RAISE e_dataAntiguitat_erronia;
      END IF;


      -- SI NO HI HA ERRORS, INSERTEM A LA TAULA
	IF p_idClient <> 0 THEN
        INSERT INTO CLIENT VALUES (p_idClient, p_NIF, p_tipusClient, p_nom, p_cognom1, p_cognom2, p_adreca, p_codiPostal, p_localitat,p_pais, p_dataAntiguitat,dataBaixaCli);
	ELSE 
        INSERT INTO CLIENT VALUES (p_idClient, p_NIF, p_tipusClient, p_nom, p_cognom1, p_cognom2, p_adreca, p_codiPostal, p_localitat,p_pais, p_dataAntiguitat,dataBaixaCli)
               RETURNING IDCLIENT INTO numIdSeq;
               ParamEntrada := '(numIdSeq :'||numIdSeq||') Params :'||paramEntrada;
        
        END IF;
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);

        COMMIT;
         
        IF P_IDCLIENT <> 0 THEN
 	DBMS_OUTPUT.PUT_LINE('REGISTRE AFEGIT A LA TAULA: CLIENT. ID('||P_IDCLIENT||')');
	ELSE
	DBMS_OUTPUT.PUT_LINE('REGISTRE AFEGIT A LA TAULA: CLIENT. ID('||numIdSeq||')');
	END IF;

      -- AVISEM SI HI HA MS D'UN NIF IGUAL (NO S ERROR, JA QUE UNA EMPRESA
      -- POT TENIR DELEGACIONS I SER CLIENTS DIFERENTS, PER AMB MATEIX NIF)
      SELECT COUNT(*) INTO numElements  FROM CLIENT
        WHERE p_NIF = CLIENT.NIF;

          IF numElements > 1
          THEN
                DBMS_OUTPUT.PUT_LINE('INFORMACIO: EL NIF ' ||p_NIF||', TE '|| numElements
                ||' ENTRADES  A LA TAULA CLIENTS');

          END IF;

EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_idClient_repetit THEN
        	RSP := 'ERROR: IDCLIENT '||P_IDCLIENT||' JA EXISTEIX A LA TAULA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_tipusClient_erroni THEN
        	RSP := 'ERROR: TIPUS CLIENT HA DE SER ''P'' = PERSONA O ''E'' = EMPRESA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_empresa_amb_cognoms THEN
        	RSP := 'ERROR: EMPRESA AMB COGNOMS';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_persona_sense_cognom1 THEN
        	RSP := 'ERROR: PERSONA HA DE TENIR COGNOM1 ALMENYS';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_dataAntiguitat_erronia THEN
        	RSP := 'ERROR: DATA DE ALTA DE CLIENT SUPERIOR A LA ACTUAL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
                ROLLBACK;

        
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		ROLLBACK;

END;
/