CREATE OR REPLACE
PROCEDURE PR_ALTA_CONTRACTE (

p_codi IN contracte.codi%TYPE,
p_idComptador IN contracte.idComptador%TYPE,
p_idClient IN contracte.idClient%TYPE,
p_KWContractats IN contracte.KWContractats%TYPE,
p_dataAlta IN contracte.dataAlta%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE

)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_CONTRACTE
--
--  Dona d'alta un contracte a la taula CONTRACTE
--  que relaciona un comptador amb un client
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la
--  TAULALOG el resultat de la operaci.
--
--
--  ENTRADA: p_CODI NUMBER(10), p_IDCOMPTADOR NUMBER(10), p_IDCLIENT NUMBER(10), 
--    p_KWCONTRACTATS NUMBER(4,1), p_DATAALTA  DATE.
--
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o si hi ha errors.
--
--  Si codi contracte = 0, s'agafa el valor seqent de la seqncia S_codi.
--  El consumAlta i consumBaixa es calcula del consum que tingui el comptador.
--  La data de baixa es posa a nul.
--
--    RSP = OK : S'inserta registre.
--
--    RSP = ERROR + motiu, si hi ha algn error.
--
--    Motius errors tractats:
--        - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--        - ERROR: CLAU CODI DUPLICADA A LA TAULA CONTRACTE.
--        - ERROR: COMPTADOR p_idComptador  NO EXISTEIX O ESTA DE BAIXA.
--        - ERROR: IDCLIENT  p_idClient  NO EXISTEIX O ESTA DE BAIXA.
--        - ERROR: JA EXISTEIX UN CONTRACTE AMB IDCOMPTADOR p_idComptador. 
--        - ERROR: COMPTADOR p_idComptador NO ADMET KW CONTRACTADA (p_KWContractats  Vs  t_KWdelModel  KW.)
--        - ERROR: DATA DE ALTA  p_dataAlta MES GRAN QUE LA DEL SISTEMA
--        - Altres
--
--******************************************************************************/


IS

    -- variables que s'insertaran a la taula contracte.
    -- les dades de baixa seran nuls.
    consumAltaContracte contracte.consumAlta%TYPE;
    consumBaixaContracte contracte.consumBaixa%TYPE := NULL;
    dataBaixaContracte contracte.dataBaixa%TYPE := NULL;

    -- variables errors
        e_camps_nuls EXCEPTION;
        e_idComptador_noexisteix  EXCEPTION;
        e_idClient_noexisteix EXCEPTION;
        e_contracte_existeix EXCEPTION;
        e_KWContractats_erroni EXCEPTION;
        e_dataAlta_erronia EXCEPTION;

    -- variables insert taulaLOG
        nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_CONTRACTE';
	paramEntrada taulaLog.paramEntrada%TYPE := 'p_codi: ' ||p_codi||', p_idComptador: ' || p_idComptador || ', p_idClient: ' || p_idClient ||
             ', p_KWContractats : ' || p_KWContractats || ', p_dataAlta : ' || p_dataAlta ;
        paramSortida taulaLog.paramSortida%TYPE := 'RSP';

    -- variables d's divers.
    numElements number :=0;
    numIdSeq number;

    -- variable de la taula MODELCOMPT
     t_KWdelModel  CONTRACTE.KWContractats%TYPE;

BEGIN

      -- VERIFIQUEM ELS NULS DELS PARAMETRES D'ENTRADA
      IF ((p_codi is null) or (p_idComptador is null) or (p_idClient is null)
          or (p_KWContractats is null) or (p_dataAlta is null))
      THEN RAISE e_camps_nuls;
      END IF;

      -- VERIFIQUEM QUE EXISTEIX COMPTADOR I NO EST DE BAIXA
      SELECT COUNT(*) INTO numElements  FROM COMPTADOR
        WHERE p_idComptador = COMPTADOR.idComptador and COMPTADOR.dataBaixa is NULL;
      IF numElements < 1
        THEN RAISE e_idComptador_noexisteix;
      END IF;

       -- VERIFIQUEM QUE EXISTEIX CLIENT i NO EST DE BAIXA
      SELECT COUNT(*) INTO numElements  FROM CLIENT
        WHERE p_idClient = CLIENT.idClient and CLIENT.dataBaixa IS NULL;
      IF numElements < 1
        THEN RAISE e_idClient_noexisteix;
      END IF;

      -- VERIFIQUEM QUE ELS KW CONTRACTATS SON SUPORTATS PEL MODEL DE COMPTADOR
      SELECT MODELCOMPT.maxKW INTO t_KWdelModel FROM MODELCOMPT WHERE MODELCOMPT.idModel =
            (SELECT COMPTADOR.idModel FROM COMPTADOR WHERE COMPTADOR.idComptador = p_idComptador);

      IF p_KWContractats > t_KWdelModel
          THEN RAISE e_KWContractats_erroni;
      END IF;

      -- VERIFIQUEM QUE DATA ALTA NO SIGUI MS GRAN QUE LA DATA ACTUAL
      IF p_dataAlta > sysdate
        THEN RAISE e_dataAlta_erronia;
      END IF;

     -- VERIFIQUEM QUE EL COMPTADOR NO TINGUI UN CONTRACTE EN VIGENCIA
     SELECT COUNT(*) INTO numElements FROM CONTRACTE
       WHERE CONTRACTE.idComptador = p_idComptador AND dataBaixa is null;
     IF numElements > 0
        THEN RAISE e_contracte_existeix;
     END IF;


     -- ASSIGNEM AL CONTRACTE EL consumAltaContracte DES DE COMPTADOR.consumAbsolut
     -- Es posa al contracte el consum del comptador en el moment de l'alta
     SELECT COMPTADOR.consumAbsolut INTO consumAltaContracte FROM COMPTADOR
      WHERE p_idComptador = COMPTADOR.idComptador;


     -- SI NO HI HA ERRORS, INSERTEM A LA TAULA CONTRACTE
     -- si p_codi = 0, anotem el nm. de seqncia assignat per saber amb
     -- quin codi s'ha donat d'alta.
      IF p_codi <> 0 THEN
      INSERT INTO CONTRACTE VALUES (p_codi, p_idComptador, p_idClient, p_KWContractats, p_dataAlta, dataBaixaContracte, consumAltaContracte, consumBaixaContracte);
      ELSE
      INSERT INTO CONTRACTE VALUES (p_codi, p_idComptador, p_idClient, p_KWContractats, p_dataAlta, dataBaixaContracte, consumAltaContracte, consumBaixaContracte)
        RETURNING codi INTO numIdSeq;
        ParamEntrada := '(numIdSeq :'||numIdSeq||') Params :'||paramEntrada;
      END IF;
      
      RSP := 'OK';
      PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);

      COMMIT;
	DBMS_OUTPUT.PUT_LINE('    REGISTRE AFEGIT A LA TAULA: CONTRACTE.');
      IF P_CODI <> 0 THEN
	DBMS_OUTPUT.PUT_LINE('    CODICONTRACTE-IDCOMPTADOR-IDCLIENT ('||p_codi||'-'||P_idcomptador||'-'||p_idClient||')');
      ELSE
	DBMS_OUTPUT.PUT_LINE('    CODICONTRACTE-IDCOMPTADOR-IDCLIENT ('||numIdSeq||'-'||P_idcomptador||'-'||p_idClient||')');
	END IF;
     
EXCEPTION

	WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_idComptador_noexisteix THEN
                RSP := 'ERROR: COMPTADOR '|| p_idComptador || ' NO EXISTEIX O ESTA DE BAIXA';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_idClient_noexisteix THEN
                RSP := 'ERROR: IDCLIENT '|| p_idClient || ' NO EXISTEIX O ESTA DE BAIXA';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_contracte_existeix THEN
                RSP := 'ERROR: JA EXISTEIX UN CONTRACTE AMB IDCOMPTADOR ' || p_idComptador ;
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_KWContractats_erroni THEN
                RSP := 'ERROR: COMPTADOR '||p_idComptador||' NO ADMET KW CONTRACTADA (' || p_KWContractats||' Vs '||t_KWdelModel||' KW.)';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN e_dataAlta_erronia  THEN
                RSP := 'ERROR: DATA DE ALTA '|| p_dataAlta||' MES GRAN QUE LA DEL SISTEMA '|| sysdate;
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

        WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

        WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU CODI DUPLICADA A LA TAULA CONTRACTE.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_ALTA_CONTRACTE'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		ROLLBACK;

END;
/