CREATE OR REPLACE
PROCEDURE PR_ALTA_LECTURA (

p_idLectura IN lectures.idLectura%TYPE,
p_idComptador IN lectures.idComptador%TYPE,
p_dataLectura IN lectures.dataLectura%TYPE,
p_lecturaActual IN lectures.lecturaActual%TYPE,
p_tipusLectura IN lectures.tipusLectura%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

--/*****************************************************************************
--  PROCEDIMENT:  PR_ALTA_LECTURA
--
--  Dona d'alta una lectura d'un comptador. S EL PROCEDIMENT MS IMPORTANT.
--  Abans d'insertar es fan una serie de verificacions, entre elles:
--      * La data lectura ha de ser superior a la ltima del comptador
--          (les lectures han de ser seqencials en el temps).
--      * El consum pot ser menor a l'anterior (abonament), per la quantitat
--          que es rebr com a parmetre ha de ser positiva (lectura que posa al
--          comptador)
--      * Amb el consum de la lectura, no s'ha de sobrepassar el mxim KWh
--        anual que poden subministrar tant les Centrals de distribuci, com
--        les lnies i les centrals de Producci.
--
--  Amb la lectura s'actualitzaran camps de la taula comptador, de tal manera que:
--      - Primer es verifica que el comptador existeixi i agafen els valors
--	de consumMensual i dataLectura (que ser consumAnt i dataLecturaAnt en
--	aquesta nova lectura)
--      Amb la lecturaActual la sumen a lecturaAnt i ser el consumAbsolut de Comptador.
--      Amb la datalectura updatem COMPTADOR.dataLectura per posar la actual.
--
--  Si la data de lectura canvia d'any, es reinicia els KWh subministrats per CDist
--  de la connexi comptador - Central Distribuci.
--
--  La lectura, s l'element que far que s'actualitzin els histrics de 
--  producci i els mduls estadstics.
--
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la
--  TAULALOG el resultat de la operaci.
--
--  ENTRADA : P_IDLECTURA NUMBER(10), p_IDCOMPTADOR NUMBER(10), p_DATALECTURA  DATE,  
--     p_LECTURAACTUAL NUMBER(8,1), p_TIPUSLECTURA  CHAR(1) .
--  SORTIDA : RSP indicant si ha anat correcte o si ha hagut errors.
--
--   El parmetre de idLectura, si s zero, s'agafa de la seqncia.
--   Els valors de dataLecturaAnt, lecturaAnt i consumLectura es calcularan.
--   El valor 'esborrat' es posa a zero a ser procediment d'alta.
--
--   RSP = OK : S'inserta registre a la taula LECTURES.
--
--    RSP = ERROR + motiu, si hi ha algun error.
--
--    Motius Error :
--	  - ERROR: HI HA NULS ALS PARAMETRES D'ENTRADA.
--        - ERROR: CLAU IDLECTURA (' || p_idLectura|| ') DUPLICADA A LA TAULA LECTURES.
--  	  - ERROR:  IDCOMPTADOR NO EXISTEIX O ESTA DE BAIXA A LA TAULA COMPTADOR.
--	  - ERROR:  LECTURA NEGATIVA. (no confondre amb consum negatiu)
--	  - ERROR: DATA DE LECTURA ACTUAL p_dataLectura  MENOR A DATA LECTURA ANTERIOR  t_dataLecturaAnt.
--		- data lectura actual d'aquell comptador ha de ser  ms gran a la de la lectura anterior d'aquell comptador.
--	  - ERROR: TIPUS LECTURA p_tipusLectura  NO ES NI (P)RESENCIAL NI (T)ELEMATICA.
--        - ERROR: COMPTADOR p_idComptador, NO TE LINIA ACTIVA FINS C-PROD.
--        - ERROR: COMPTADOR NOMES POT REBRE D'UNA LINIA ACTIVA.
--        - ERROR: LECTURA PERTANY A COMPTADOR  p_idComptado QUE NO TE CONTRACTE. 
--        - ERROR: AMB EL CONSUM DE LA LECTURA ES SOBREPASSARIA EL MAXIM.  (De la C.Dist, Linia o C.Prod).
--        - Altres.
--
--  NO S MOTIU D'ERROR que una lectura sigui menor a la anterior, ja que
--	poT haver-hi abonaments o equivocacions en les lectures i s'ha de poder
--	restar el consum al client per no facturar-li.
--
--******************************************************************************/

IS

  -- variables que s'insertarn a la taula LECTURES, algunes es calcularan.
  esborratLect lectures.esborrat%TYPE := 0;
  t_dataLecturaAnt lectures.dataLecturaAnt%TYPE;
  t_lecturaAnt lectures.lecturaAnt%TYPE;
  consumLect lectures.consumLectura%TYPE;

  -- variables errors
  e_idComptador_noexisteix EXCEPTION;
  e_camps_nuls EXCEPTION;
  e_lectura_negativa EXCEPTION;
  e_tipusLectura_erroni EXCEPTION;
  e_dataLectura_erronia EXCEPTION;
  e_campEsborrat_erroni EXCEPTION;
  e_noContracte EXCEPTION;
  e_sobrepassaMaxKW EXCEPTION;
  e_noLiniaActiva EXCEPTION;
  e_liniadoble EXCEPTION;

  -- variables insert taulaLOG
  nomProcediment taulaLog.procExecutat%TYPE := 'PR_ALTA_LECTURA';
  paramEntrada taulaLog.paramEntrada%TYPE := 'P_idLectura :'
      || p_idLectura|| ', p_idComptador: '|| p_idComptador||', p_dataLectura: '|| p_dataLectura
      || ', p_lecturaActual : ' || p_lecturaActual ||', p_tipusLectura: '|| p_tipusLectura;
  paramSortida taulaLog.paramSortida%TYPE := 'RSP';

  -- variables d's divers. Comptadors.. etc.
  numElements number := 0;
  cDist conncomptador.IDCDIST%TYPE;
  Linia CONNLINIA.idLinia%TYPE;
  sumKW CONNCOMPTADOR.KWHCONNEXIO%TYPE :=0;
  maxKW CONNCOMPTADOR.KWHCONNEXIO%TYPE :=0;
  numIdSeq number;

  -- variables procedents de selects per veure la procedncia del subministre.
  t_idCDist CONNLINIA.idCDist%TYPE;
  t_idCProd CONNLINIA.idCProd%TYPE;
  t_idLinia CONNLINIA.idLinia%TYPE;

BEGIN
      -- VERIFIQUEM QUE NO HI HAGI NULS ALS PARAMETRES D'ENTRADA
      IF ((p_idLectura is null) or (p_idComptador is null) or  (p_dataLectura is null) or (p_lecturaActual is null)
         or (p_tipusLectura is null) or (esborratLect is null))
        THEN RAISE e_camps_nuls;
      END IF;

      -- VERIFIQUEM QUE EXISTEIXI COMPTADOR i NO EST DE BAIXA

      SELECT COUNT(*) INTO numElements FROM COMPTADOR
        WHERE p_idComptador = COMPTADOR.idComptador AND COMPTADOR.dataBaixa IS NULL;
      IF numElements = 0
        THEN RAISE e_idComptador_noexisteix;
      END IF;

      -- DEL COMPTADOR AGAFEM ELS VALORS DE CONSUM ABSOLUT I DATA LECTURA ANTERIORS.
      -- i calculem el consum de la lectura
      SELECT COMPTADOR.consumAbsolut, COMPTADOR.dataLectura INTO t_lecturaAnt, t_dataLecturaAnt
         FROM COMPTADOR
         WHERE p_idComptador = COMPTADOR.idComptador;

        consumLect := p_lecturaActual - t_lecturaAnt;

      -- EL NUMERO DE KWH ANOTATS DEL COMPTADOR SEMPRE ES POSITIU
      -- (NO CONFONDRE LECTURA NEGATIVA AMB CONSUM NEGATIU)
      IF p_lecturaActual < 0
         THEN RAISE e_lectura_negativa;
      END IF;

        --VERIFIQUEM QUE LA DATA DE LECTURA SIGUI POSTERIOR A LA ANTERIOR.
      IF t_dataLecturaAnt > p_dataLectura
        THEN RAISE e_dataLectura_erronia;
      END IF;

        -- VERIFIQUEM QUE EL TIPUS DE LECTURA SIGUI CORRECTE (T, P)
      IF p_tipusLectura <> 'P' and p_tipusLectura <> 'T'
        THEN RAISE e_tipusLectura_erroni;
      END IF;

      -- VERIFIQUEM QUE EL COMPTADOR PERTANY A UN CONTRACTE ACTIU
      -- ENTRE COMPTADOR - CLIENT.
      SELECT COUNT(*) INTO numElements
        FROM CONTRACTE
        WHERE CONTRACTE.idComptador = p_idComptador
          AND CONTRACTE.dataBaixa is NULL;

      IF numElements = 0
        THEN RAISE e_noContracte;
      END IF;

       -- VERIFIQUEM QUE LA LECTURA PERTANY  A UNA (I NOMES UNA) LINIA ACTIVA FINS CPROD
      SELECT COUNT(*) INTO numElements
        FROM conncomptador, connlinia
        WHERE conncomptador.idcdist = connlinia.idcdist AND p_idComptador = conncomptador.idComptador
              AND conncomptador.estat = 1 AND connlinia.estat =1
              AND CONNCOMPTADOR.TIPUSCONN <> 'B' AND CONNLINIA.TIPUSCONN <> 'B';

      IF numElements = 0
        THEN RAISE e_noLiniaActiva;
      ELSE IF numElements >1
              THEN RAISE e_liniadoble ;
      ELSE
      --SELECCIONEM ELS CAMPS DELS ELEMENTS D'ON REP LA ENERGIA PER UTILITZAR MS ENDAVANT
          SELECT connlinia.idcprod, connlinia.idLinia, conncomptador.idcdist
            INTO  t_idcprod, t_idLinia, t_idCDist
            FROM conncomptador, connlinia
            WHERE conncomptador.idcdist = connlinia.idcdist and p_idComptador = conncomptador.idComptador
            AND conncomptador.estat = 1 AND connlinia.estat =1
            AND CONNCOMPTADOR.TIPUSCONN <> 'B' AND CONNLINIA.TIPUSCONN <> 'B';
           END IF;
       END IF;

       -- ABANS D'INSERTAR, S'HA DE VERIFICAR QUE AMB EL CONSUM QUE S'ANOTAR
       -- NO ES SUPERA EL MXIM ANUAL DE KWH QUE SUPORTA TANT LA CENTRAL DE
       -- DISTRIBUCI, COM LA LNIA, COM LA CENTRAL DE PRODUCCI DE LA QUE
       -- PENJA EL CONSUM
       IF   F_VERIFICAR_MAXIMSKWH(p_dataLectura, consumLect, p_idComptador, t_idCDist, t_idLinia, t_idCProd,RSP)= TRUE
       THEN RAISE e_sobrepassaMaxKw;
       END IF;

 -------- SI NO HI HA ERRORS, INSERTEM A LECTURES, 
      -- si idLectura = 0, anotem al log el nmero de seqncia amb el que
      -- s'haur donat d'alta com a idLectura.

      IF p_idLectura <> 0 THEN
      INSERT INTO LECTURES VALUES (p_idLectura,p_idComptador, t_dataLecturaAnt, p_dataLectura, t_lecturaAnt,
          p_lecturaActual, consumLect, p_tipusLectura,esborratLect);
      ELSE
      
      INSERT INTO LECTURES VALUES (p_idLectura,p_idComptador, t_dataLecturaAnt, p_dataLectura, t_lecturaAnt,
          p_lecturaActual, consumLect, p_tipusLectura,esborratLect)
      RETURNING IDLECTURA INTO numIDSeq;
      paramEntrada := '(numIdSeq :'||numIdSeq||') Params :'||paramEntrada;
      END IF;
      

       --DBMS_OUTPUT.PUT_LINE('LECTURA AFEGIDA IDCOMPTADOR ' ||p_idComptador|| '. DATA: '|| p_dataLectura|| '. LECTURA :' ||p_lecturaActual|| '. CONSUM:'||consumLect);

        -- modifiquem els valors a COMPTADOR de consums i dates lectures
       UPDATE COMPTADOR SET COMPTADOR.consumAbsolut = p_lecturaActual, COMPTADOR.consumMensual = consumLect, COMPTADOR.dataLectura = p_dataLectura
       WHERE COMPTADOR.idComptador = p_idComptador;

       -- SI HA CANVIAT D'ANY, LLAVORS REINICIEM ELS CONSUMS ACUMULATS A CONNCOMPTADOR
       IF (TO_CHAR(t_dataLecturaAnt, 'yyyy'))<>(TO_CHAR(p_datalectura, 'yyyy'))  THEN
          -- DBMS_OUTPUT.PUT_LINE('CANVI D''ANY. REINICIEM KWH A CONNCOMPTADOR');
            UPDATE CONNCOMPTADOR
            SET CONNCOMPTADOR.KWHCONNEXIO = consumLect
            WHERE CONNCOMPTADOR.IDCOMPTADOR = p_idComptador
                AND CONNCOMPTADOR.ESTAT = 1 AND CONNCOMPTADOR.tipusConn <> 'B';

       ELSE
          -- SI NO HA CANVIAT D'ANY, AFEGIM A CONNCOMPTADOR ELS KWH CONSUMITS
          UPDATE CONNCOMPTADOR
          SET CONNCOMPTADOR.KWHCONNEXIO = CONNCOMPTADOR.KWHCONNEXIO + consumLect
          WHERE CONNCOMPTADOR.IDCOMPTADOR = p_idComptador
            AND CONNCOMPTADOR.ESTAT = 1 AND CONNCOMPTADOR.tipusConn <> 'B';

      END IF;

      RSP := 'OK';
       
       PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);

        -- ACTUALITZEM HISTORICCPROD
        PR_ACTUALITZAR_HISTORICCPROD(p_dataLectura,p_idComptador, consumLect,RSP);

       -- ACTUALITZEM MODULS ESTADISTICS

       PR_ACTUALITZAR_ME1(t_idCProd, consumLect,RSP);
       PR_ACTUALITZAR_ME2(t_idlinia, p_dataLectura, p_idComptador, consumLect,RSP);
       PR_ACTUALITZAR_ME3(t_idlinia,RSP);
       PR_ACTUALITZAR_ME4(RSP);
       PR_ACTUALITZAR_ME5(RSP);
       PR_ACTUALITZAR_ME6(p_idComptador, p_lecturaActual, RSP);
       PR_ACTUALITZAR_ME7(RSP);
     
       COMMIT;
       
EXCEPTION

      WHEN e_camps_nuls THEN
        	RSP := 'ERROR: HI HA NULS ALS PARAMETRES D''ENTRADA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_tipusLectura_erroni THEN
        	RSP := 'ERROR: TIPUS LECTURA '''|| p_tipusLectura || ''' NO ES NI (P)RESENCIAL NI (T)ELEMATICA';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_idComptador_noexisteix THEN
        	RSP := 'ERROR:  IDCOMPTADOR NO EXISTEIX O ESTA DE BAIXA A LA TAULA COMPTADOR';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_lectura_negativa THEN
        	RSP := 'ERROR:  LECTURA NEGATIVA.';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_dataLectura_erronia THEN
        	RSP := 'ERROR: DATA DE LECTURA ACTUAL '|| p_dataLectura || ' MENOR A DATA LECTURA ANTERIOR '|| t_dataLecturaAnt;
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_sobrepassaMaxKw THEN
        	RSP := 'ERROR: AMB EL CONSUM DE LA LECTURA ES SOBREPASSARIA EL MAXIM';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_noContracte THEN
        	RSP := 'ERROR: LECTURA PERTANY A COMPTADOR ' || p_idComptador||', QUE NO TE CONTRACTE ';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_noLiniaActiva THEN
        	RSP := 'ERROR: COMPTADOR ' || p_idComptador||', NO TE LINIA ACTIVA FINS C-PROD ';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN e_liniaDoble THEN
        	RSP := 'ERROR: COMPTADOR NOMES POT REBRE D''UNA LINIA ACTIVA';
      		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

      WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

      WHEN DUP_VAL_ON_INDEX THEN
        	RSP := 'ERROR: CLAU IDLECTURA (' || p_idLectura|| ') DUPLICADA A LA TAULA LECTURES.';
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
		ROLLBACK;

      WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR PR_ALTA_LECTURA - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		ROLLBACK;

END;
/