CREATE OR REPLACE
PROCEDURE PR_BAIXA_COMPTADOR (

p_idComptador IN comptador.idComptador%TYPE,
p_dataBaixa IN comptador.dataBaixa%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_BAIXA_COMPTADOR
--    
--  Dona de baixa un comptador en la taula COMPTADOR.
--  
--  Tant si hi ha errors com si no, s'afegeix a la 
--  TAULALOG el resultat de la operaci-
--    
--  ENTRADA : P_IDCOMPTADOR   NUMBER(10)  , p_dataBaixa   DATE  
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--  Si parmetre DataBaixa rebut s null, s'agafa del sistema.
--  
--    RSP = OK
--          - Es posa la data de baixa a Comptador, a ConnComptador i als contractes
--            que hi hagi amb aquell comptador.
--          - Es posa com a esborrat=1 totes les lectures d'aquell comptador.
--    
--    RSP = ERROR + motiu, si hi ha algún error.
--  
--    Motius Error :
--        - ERROR: ID COMPTADOR p_idComptador NO EXISTEIX O JA ESTA DE BAIXA.
--        - ERROR: PARAMETRE IDCOMPTADOR NUL. 
--        - ERROR: CONTRACTE NO EXISTEIX O JA ESTA DE BAIXA.
--        - ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA DE LA ULTIMA LECTURA.
--        - Altres.
--
******************************************************************************/



IS
    -- variables errors
    e_idComptador EXCEPTION;
    e_idComptador_nul EXCEPTION;
    e_dataBaixa_erronia EXCEPTION;
    e_contracte EXCEPTION;
       
    -- variables insert taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_BAIXA_COMPTADOR';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idComptador: ' || p_idComptador || ', p_dataBaixa:'|| p_dataBaixa;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
    -- variable de data de baixa que es posar al registre.    
    dataBaixaCompt  COMPTADOR.DATABAIXA%type;
    
    -- variable amb la ltima data lectura del comptador.
    t_dataLectura  COMPTADOR.DATABAIXA%type;
    
    -- variables d's divers.
    numElements number := 0;
    
    -- variable amb el consum del comptador en el moment de la baixa
    t_consumAbsolut COMPTADOR.CONSUMABSOLUT%type;
    
BEGIN
        
     -- SI EL PARMETRE DE DATA BAIXA S NUL, LA AGAFEM DEL SISTEMA     
    IF p_dataBaixa is null 
      THEN dataBaixaCompt := sysdate;
    ELSE dataBaixaCompt := p_dataBaixa;
    END IF;
                
    -- VERIFIQUEM QUE EL PARMETRE IDCOMPTADOR REBUT NO S NULL
    IF (p_idComptador is null) 
      THEN RAISE e_idComptador_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A COMPTADOR I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM COMPTADOR 
      WHERE p_idComptador = COMPTADOR.idComptador AND COMPTADOR.dataBaixa is NULL;
	 	
    IF numElements <>1
      THEN RAISE e_idComptador;
    END IF;
        
       
      -- VERIFIQUEM QUE EXISTEIX A CONTRACTE I QUE NO EST DONAT DE BAIXA
      SELECT COUNT(*) INTO numElements 
        FROM CONTRACTE 
        WHERE p_idComptador = CONTRACTE.idComptador AND CONTRACTE.dataBaixa is NULL;
              
      IF numElements <> 1
        THEN RAISE e_contracte;
      END IF;
        
        -- VERIFIQUEM QUE dataBaixaCompt NO SIGUI MENOR QUE LA DATA DE LA LTIMA LECTURA (incongruent)
        SELECT comptador.dataLectura INTO t_dataLectura 
          FROM comptador 
          WHERE comptador.idComptador = p_idComptador;
        t_dataLectura := t_dataLectura-1;
        IF greatest(t_dataLectura,dataBaixaCompt) = t_dataLectura
          THEN RAISE e_dataBaixa_erronia;
        END IF;
        
	-- actualitzem el camp dataBaixa de Comptador
        SELECT COMPTADOR.CONSUMABSOLUT INTO t_consumAbsolut FROM comptador WHERE COMPTADOR.idComptador = p_idComptador;
	UPDATE COMPTADOR SET COMPTADOR.dataBaixa = dataBaixaCompt WHERE COMPTADOR.idComptador = p_idComptador;
        
        -- Marquem les lectures del comptador com esborrades.
        UPDATE LECTURES SET LECTURES.ESBORRAT = 1 WHERE LECTURES.idComptador = p_idComptador;
        
        -- Marquel el contracte com a Baixa i anotem el consum en el moment de la baixa.
        UPDATE CONTRACTE 
          SET CONTRACTE.dataBAixa = dataBaixaCompt, CONTRACTE.consumBaixa = t_consumAbsolut
          WHERE CONTRACTE.idComptador = p_idComptador;
          
        -- Marquem la connexi de comptador (comptador - C. dist) de Baixa.
        UPDATE CONNCOMPTADOR 
          SET CONNCOMPTADOR.tipusConn = 'B', CONNCOMPTADOR.estat=0 
          WHERE CONNCOMPTADOR.idComptador = p_idComptador;
	        
        COMMIT;
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        
        DBMS_OUTPUT.PUT_LINE('COMPTADOR ID '||p_idComptador||' DONAT DE BAIXA');
        DBMS_OUTPUT.PUT_LINE('(AIXI COM ELS CONTRACTES, LECTURES I CONNEXIONS ASSOCIADES)');
	
EXCEPTION

	WHEN e_idComptador_nul THEN
        	RSP := 'ERROR: PARAMETRE IDCOMPTADOR NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idComptador THEN
        	RSP := 'ERROR: ID COMPTADOR ' ||p_idComptador||' NO EXISTEIX O JA ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_contracte THEN
        	RSP := 'ERROR: CONTRACTE NO EXISTEIX O JA ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;       
                
        WHEN e_dataBaixa_erronia THEN
        	RSP := 'ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA DE LA ULTIMA LECTURA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;

	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
		
END;
/