CREATE OR REPLACE
PROCEDURE PR_BAIXA_CONTRACTE (

p_codi IN CONTRACTE.codi%TYPE,
p_dataBaixa IN CONTRACTE.dataBaixa%TYPE,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)


/*****************************************************************************
--  PROCEDIMENT:  PR_BAIXA_CONTRACTE
--    
--  Dona de baixa un contracte entre un client i un comptador.
--  
--  Tant si hi ha errors com si no, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_CODI  NUMBER(10)  , p_DATABAIXA  DATE .
--  SORTIDA : RSP VARCHAR2(300) indicant si ha anat correcte o si ha hagut errors.
--  
--  Si parmetre DataBaixa rebut s null, s'agafa del sistema.
--  
--    RSP = OK
--          - Es posa la data de baixa a CONTRACTE. 
--          - Es posa el consum del comptador del contracte en moment de la baixa.
--    
--   RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--       - ERROR: CODI CONTRACTE p_codi NO EXISTEIX O JA ESTA DE BAIXA.
--       - ERROR: PARAMETRE CODI NUL. 
--       - ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA D'ALTA.
--       - Altres.
--       
******************************************************************************/



IS
    -- variables d'error
    e_codi_noexisteix EXCEPTION;
    e_codi_nul EXCEPTION;
    e_dataBaixa_erronia EXCEPTION;

    -- variables de insert a taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_BAIXA_CONTRACTE';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_codi: ' || p_codi || ',  p_dataBaixa:'|| p_dataBaixa;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';

    -- data baixa a posar al registre
    dataBaixaContracte  CONTRACTE.dataBaixa%TYPE;
    
    -- data alta del contracte
    t_dataAlta  CONTRACTE.dataAlta%TYPE;
    
    -- variable d's divers.
    numElements number := 0;
        
BEGIN
    -- SI EL PARMETRE DE DATA BAIXA S NULL, S'AGAFA DEL SISTEMA
    IF p_dataBaixa is null 
      THEN dataBaixaContracte := sysdate;
    ELSE dataBaixaContracte := p_dataBaixa;
    END IF;
                
    -- VERIFIQUEM QUE EL PARAMETRE IDCLIENT REBUT NO S NULL
    IF (p_codi is null) 
      THEN RAISE e_codi_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX EL CONTRACTE I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM CONTRACTE
      WHERE p_codi = CONTRACTE.CODI AND CONTRACTE.dataBaixa is NULL;
	 	
    IF numElements <> 1
      THEN RAISE e_codi_noexisteix;
    END IF;
   
    -- VERIFIQUEM QUE dataBaixaContracte NO SIGUI MENOR QUE LA DATA D'ALTA
    SELECT CONTRACTE.dataAlta INTO t_dataAlta 
      FROM CONTRACTE
      WHERE CONTRACTE.codi = p_codi;
        
      t_dataAlta := t_dataAlta-1;
    
    IF greatest(t_dataAlta,dataBaixaContracte) = t_dataAlta
      THEN RAISE e_dataBaixa_erronia;
    END IF;
        
   
        
	-- SI NO HI HA ERRORS, DONEM DE BAIXA EL CONTRACTE POSANT LA DATA BAIXA
        UPDATE CONTRACTE SET CONTRACTE.dataBaixa = dataBaixaContracte WHERE CONTRACTE.codi = p_codi;
        
        -- POSEM EL CONSUM DEL COMPTADOR EN EL CONTRACTE EN EL MOMENT DE LA BAIXA
        UPDATE CONTRACTE SET CONTRACTE.consumBaixa = (SELECT COMPTADOR.consumAbsolut 
                    FROM COMPTADOR WHERE COMPTADOR.idComptador = CONTRACTE.idComptador)
                    WHERE CONTRACTE.codi = p_codi;
                                                                                                             
        
        COMMIT;
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        DBMS_OUTPUT.PUT_LINE('CONTRACTE AMB CODI '||p_codi||' DONAT DE BAIXA');
	
EXCEPTION

	WHEN e_codi_nul THEN
        	RSP := 'ERROR: PARAMETRE CODI NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_codi_noexisteix THEN
        	RSP := 'ERROR: CODI CONTRACTE ' ||p_codi||' NO EXISTEIX O JA ESTA DE BAIXA';
                PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_dataBaixa_erronia THEN
        	RSP := 'ERROR: DATA BAIXA HA DE SER IGUAL O MAJOR A LA DATA D''ALTA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
                Rollback;

                      
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR - PR_BAIXA_CONTRACTE '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;

END;
/