CREATE OR REPLACE
PROCEDURE PR_MODIFICA_CENTRALDIST (

p_idCDist IN CENTRALDIST.idCDist%TYPE,
p_adreca IN CENTRALDIST.adreca%TYPE := '*',
p_codiPostal IN CENTRALDIST.codiPostal%TYPE := '*',
p_localitat IN CENTRALDIST.localitat%TYPE  := '*',
P_pais  IN CENTRALDIST.pais%TYPE  := '*',
p_maxKWh IN CENTRALDIST.maxKWh%TYPE := -1,

RSP OUT NOCOPY TAULALOG.RSP%TYPE
)

/*****************************************************************************
--  PROCEDIMENT:  PR_MODIFICA_CENTRALDIST
--    
--  Modifica les dades d'una central de distribuci segons el seu idCDist.
--  Noms es pot canviar l'adreca, i el mximKWh anual que pot suportat 
--  ja que la resta de dades no sn modificables  a efectes de coherncia 
--  (any fabricaci, idmodel, consums...).
--  
--  En cas que es modifiqui el maxKWh anual, el sistema donar un missatge
--  d'avs si el nou valor s menor a l'anterior ja que pot fer que amb la
--  propera lectura es sobrepassi.
--  
--  Tant si hi ha errors com si insertem el registre a la taula, s'afegeix a la 
--  TAULALOG el resultat de la operaci.
--    
--  ENTRADA : P_IDCDIST  NUMBER(10), p_ADRECA VARCHAR(30), p_CODIPOSTAL CHAR(5), 
--      p_ LOCALITAT VARCHAR2(30), p_PAIS VARCHAR2(20) , p_MAXKWH  NUMBER(12,1) .
--            
--  SORTIDA : RSP varchar2(300) indicant si ha anat correcte o amb errors.
--  
--  Si un camp dels parmetres no es vol modificar, en el seu lloc als 
--  parmetres d'entrada s'ha de posar null.
--  
--    RSP = OK
--          
--    RSP = ERROR + motiu, si hi ha algn error.
--  
--    Motius Error :
--         - ERROR: PARAMETRE idCDist NUL.
--         - ERROR: ID CENTRALDIST p_idCDist NO EXISTEIX ESTA DE BAIXA.
--         - ERROR: PARAMETRE MAXKWH REBUT ES NEGATIU.
--         - Altres
******************************************************************************/

IS
    
    e_idCDist EXCEPTION;
    e_idCDist_nul EXCEPTION;
    e_maxKWh_negatiu EXCEPTION;
       
    -- variables insert taulaLOG   
    nomProcediment taulaLog.procExecutat%TYPE := 'PR_MODIFICA_CENTRALDIST';
    paramEntrada taulaLog.paramEntrada%TYPE := 'P_idCDist: ' || p_idCDist || ', p_adreca: '|| p_adreca
              ||', p_codiposta: '|| p_codipostal||', p_localitat: '||p_localitat
              || ', p_pais: '||p_pais|| ', p_maxKWh: '||p_maxKWh;
    paramSortida taulaLog.paramSortida%TYPE := 'RSP';
	
    -- variables d's divers.
    numElements number := 0;
    maxKWh_temp CENTRALDIST.MAXKWH%TYPE;
   
BEGIN
        
    -- VERIFIQUEM QUE EL PARAMETRE idCDist REBUT NO s NULL
    IF (p_idCDist is null) 
      THEN RAISE e_idCDist_nul;
    END IF;
       
    -- VERIFIQUEM QUE EXISTEIX A CENTRALDIST I QUE NO EST DONAT DE BAIXA
    SELECT COUNT(*) INTO numElements 
      FROM CENTRALDIST 
      WHERE p_idCDist = CENTRALDIST.idCDist AND CENTRALDIST.dataBaixa is NULL;
	 	
    IF numElements <>1
      THEN RAISE e_idCDist;
    END IF;
        
    
    -- si parmetre maxkwh s negatiu, llencem error
    IF p_maxKWh < 0 THEN 
      RAISE e_maxKWh_negatiu;
    END IF;
    
    -- si el valor maxkwh que es vol posar s menor al que hi ha
    -- avisem a l'usuari ja que es pot sobrepassar ms fcilment el mxim.
    SELECT CENTRALDIST.MAXKWH INTO MAXKWH_TEMP 
          FROM CENTRALDIST
          WHERE CENTRALDIST.IDCDIST = p_idcDist;
          
    IF MAXkWH_TEMP > p_maxKWh
      THEN DBMS_OUTPUT.PUT_LINE('AVIS : MAXKWH NOU ES MENOR AL QUE HI HAVIA');
    END IF;
    
      
    -- SI NO HI HA ERRORS, UPDATEM I ANOTEM AL LOG.
    -- NOMS CANVIEN DE VALOR ELS CAMPS DELS PARMETRES DIFERENTS DE NULL.
    
    UPDATE CENTRALDIST SET adreca = DECODE(p_adreca,'*', adreca , NULL, adreca , p_adreca),
                         codiPostal = DECODE (p_codiPostal, '*', codiPostal, Null , codiPostal, p_codiPostal),
                         localitat = DECODE (p_localitat, '*', localitat, Null , localitat, p_localitat),
                         pais = DECODE (p_pais, '*', pais, Null , pais, p_pais),
                         maxKWh = DECODE (p_maxKWh, -1, maxKWh, Null , maxKWh, p_maxKWh)
                     WHERE CENTRALDIST.idCDist = p_idCDist;
     
        COMMIT;
        
        RSP := 'OK';
	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);
        
        DBMS_OUTPUT.PUT_LINE('CENTRALDIST ID '||p_idCDist||' MODIFICAT');
        
	
EXCEPTION

	WHEN e_idCDist_nul THEN
        	RSP := 'ERROR: PARAMETRE idCDist NUL';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
                
        WHEN e_idCDist THEN
        	RSP := 'ERROR: ID CENTRALDIST ' ||p_idCDist||' NO EXISTEIX O JA ESTA DE BAIXA';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
       WHEN e_maxKWh_negatiu THEN
        	RSP := 'ERROR: PARAMETRE MAXKWH REBUT ES NEGATIU';
        	PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);
		Rollback;
       
	WHEN STORAGE_ERROR THEN
		RSP := 'ERROR: ERROR AL EMMAGATZEMAR'||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP,  sysdate);	
		ROLLBACK;
                
	WHEN OTHERS THEN
		RSP := 'ERROR: ERROR SENSE TRACTAR -PR_MODIFICA_CENTRALDIST  '||SQLCODE||'-'||SQLERRM;
		PR_INSERT_LOG( nomProcediment,paramEntrada, paramSortida, RSP, sysdate);	
		ROLLBACK;
		
END;
/